==> roboduck-0.3.3.tar.gz: roboduck-0.3.3/roboduck/magic.py <==
"""GPT-powered rough equivalent of the `%debug` Jupyter magic. After an error
occurs, just run %duck in the next cell to get an explanation. This is very
similar to using the errors module, but is less intrusive - you only call it
when you want an explanation, rather than having to type y/n after each error.
We also provide `paste` mode, which attempts to paste a solution into a new
code cell below, and `interactive` mode, which throws you into a conversational
debugging session (technically closer to the original `%debug` magic
functionality.

Quickstart
----------
```
# cell 1
from roboduck import magic

nums = [1, 2, 3]
nums.add(4)
```

```
# cell 2
%duck
```
"""

from functools import partial
from IPython import get_ipython
from IPython.core.magic import line_magic, magics_class, Magics
from IPython.core.magic_arguments import argument, magic_arguments, \
    parse_argstring
import sys
import warnings

from roboduck.debug import DuckDB, CodeCompletionCache
from roboduck.ipy_utils import is_colab


@magics_class
class DebugMagic(Magics):
    """Enter a conversational debugging session after an error is thrown by a
    jupyter notebook code cell.

    Examples
    --------
    After a cell execution throws an error, enter this in the next cell to
    get an explanation of what caused the error and how to fix it:

    %duck

    To instead enter an Interactive conversational debugging session:

    %duck -i

    If the -p flag is present, we will try to Paste a solution into a new
    code cell upon exiting the debugger containing the fixed code snippet.

    %duck -p

    Flags can be combined:

    %duck -ip
    """

    @magic_arguments()
    @argument('-p', action='store_true',
              help='Boolean flag: if provided, try to PASTE a solution into a '
                   'new code cell below.')
    @argument('-i', action='store_true',
              help='Boolean flag: if provided, use INTERACTIVE mode. Start a '
                   'conversational debugger session and allow the user to ask '
                   'custom questions, just as they would if using '
                   'roboduck.debug.duck(). The default mode, meanwhile, '
                   'simply asks gpt what caused the error that just '
                   'occurred and then exits, rather than lingering in a '
                   'debugger session.')
    @argument('--prompt', type=str, default=None)
    @line_magic
    def duck(self, line=''):
        """Silence warnings for a cell. The -p flag can be used to make the
        change persist, at least until the user changes it again.
        """
        args = parse_argstring(self.duck, line)
        if args.prompt:
            warnings.warn('Support for custom prompts is somewhat limited - '
                          'your prompt must use the default parse_func '
                          '(roboduck.utils.parse_completion).')
        if args.p and is_colab(self.shell):
            warnings.warn('Paste mode is unavailable in google colab, which '
                          'you appear to be using. Ignoring -p flag.')
        if args.i:
            old_cls = self.shell.debugger_cls
            if args.prompt:
                new_cls = partial(DuckDB, prompt=args.prompt)
            else:
                new_cls = DuckDB
            try:
                self.shell.debugger_cls = new_cls
            except AttributeError:
                print(
                    'Roboduck is unavailable in your current ipython session. '
                    'To use it, start a new session with the command:\n\n'
                    'ipython --TerminalIPythonApp.interactive_shell_class='
                    'roboduck.shell.RoboDuckTerminalInteractiveShell\n\n'
                    '(You will also need to run `from roboduck import '
                    'magic` in the session to make the magic available.) To '
                    'make it available automatically for all '
                    'ipython sessions by default, add the following lines to '
                    'your ipython config (usually found at '
                    '~/.ipython/profile_default/ipython_config.py):\n\n'
                    'cfg = get_config()\ncfg.TerminalIPythonApp.interactive_'
                    'shell_class = roboduck.shell.'
                    'RoboDuckTerminalInteractiveShell'
                    '\ncfg.InteractiveShellApp.exec_lines = '
                    '["from roboduck import magic"]'
                )
                return
            self.shell.InteractiveTB.debugger_cls = new_cls
            self.shell.debugger(force=True)
            self.shell.debugger_cls = self.shell.InteractiveTB.debugger_cls = old_cls
        else:
            # Confine this import to this if clause rather than keeping a top
            # level import - importing this module overwrites sys.excepthook
            # which we don't necessarily want in most cases.
            # Note that this uses the `debug_stack_trace` prompt by default
            # whereas interactive mode uses `debug` by default.
            # UPDATE: we could probably use excepthook for both cases
            # (args.i = True or False) now that
            # I added interactive support in the errors module. However,
            # everything is working nicely now and I don't see a compelling
            # reason to change things at the moment.
            from roboduck import errors
            kwargs = {'auto': True, 'color': 'green'}
            if args.prompt:
                kwargs['prompt'] = args.prompt
            errors.excepthook(sys.last_type, sys.last_value,
                              sys.last_traceback, **kwargs)
            errors.disable()

        # Insert suggested code into next cell.
        if args.p and CodeCompletionCache.last_completion:
            self.shell.set_next_input(CodeCompletionCache.last_new_code,
                                      replace=False)
        CodeCompletionCache.reset_class_vars()


get_ipython().register_magics(DebugMagic)

==> roboduck-0.5.0.tar.gz: roboduck-0.5.0/roboduck/magic.py <==
"""GPT-powered rough equivalent of the `%debug` Jupyter magic. After an error
occurs, just run %duck in the next cell to get an explanation. This is very
similar to using the errors module, but is less intrusive - you only call it
when you want an explanation, rather than having to type y/n after each error.
We also provide `paste` mode, which attempts to paste a solution into a new
code cell below, and `interactive` mode, which throws you into a conversational
debugging session (technically closer to the original `%debug` magic
functionality.

Quickstart
----------
```
# cell 1
from roboduck import magic

nums = [1, 2, 3]
nums.add(4)
```

```
# cell 2
%duck
```
"""

from IPython import get_ipython
from IPython.core.magic import line_magic, magics_class, Magics
from IPython.core.magic_arguments import argument, magic_arguments, \
    parse_argstring
import sys
import warnings

from roboduck.debug import CodeCompletionCache
from roboduck.ipy_utils import is_colab


@magics_class
class DebugMagic(Magics):
    """Enter a conversational debugging session after an error is thrown by a
    jupyter notebook code cell.

    Examples
    --------
    After a cell execution throws an error, enter this in the next cell to
    get an explanation of what caused the error and how to fix it:

    %duck

    To instead enter an Interactive conversational debugging session:

    %duck -i

    If the -p flag is present, we will try to Paste a solution into a new
    code cell upon exiting the debugger containing the fixed code snippet.

    %duck -p

    Flags can be combined:

    %duck -ip
    """

    @magic_arguments()
    @argument('-p', action='store_true',
              help='Boolean flag: if provided, try to PASTE a solution into a '
                   'new code cell below. If you also choose to use '
                   'interactive mode and receive multiple replies containing '
                   'code snippets, the last snippet to be generated will be '
                   'the one that is pasted in (we make the assumption that '
                   'users will prompt for increasingly correct code rather '
                   'than the reverse).')
    @argument('-i', action='store_true',
              help='Boolean flag: if provided, use INTERACTIVE mode. Start a '
                   'conversational debugger session and allow the user to ask '
                   'custom questions, just as they would if using '
                   'roboduck.debug.duck(). The default mode, meanwhile, '
                   'simply asks gpt what caused the error that just '
                   'occurred and then exits, rather than lingering in a '
                   'debugger session.')
    @argument('--prompt', type=str, default=None)
    @line_magic
    def duck(self, line=''):
        """Silence warnings for a cell. The -p flag can be used to make the
        change persist, at least until the user changes it again.
        """
        args = parse_argstring(self.duck, line)
        if args.prompt:
            warnings.warn('Support for custom prompts is somewhat limited '
                          'at the moment - your prompt must use the default '
                          'parse_func (roboduck.utils.parse_completion).')
        if args.p and is_colab(self.shell):
            warnings.warn('Paste mode is unavailable in google colab, which '
                          'you appear to be using. Ignoring -p flag.')
        try:
            # Confine this import to this method rather than keeping a top
            # level import because importing this module overwrites
            # sys.excepthook which we don't necessarily want when
            # importing this module.
            # Note that this uses the `debug_stack_trace` prompt by default.
            # This does NOT include a user question param in the contextful
            # prompt, but in this setting we only use that method once (for our
            # automatic question) because excepthook transports us to one
            # step before the error. The user can ask followup questions but
            # the code state will not progress because we can't step through
            # any further, and thus the contextless method (which does include
            # a user question) is used from that point forward.
            # Color should be specified because errors module uses red
            # by default (whereas debug module uses green).
            from roboduck import errors
            kwargs = {'auto': True, 'interactive': args.i, 'color': 'green'}
            if args.prompt:
                kwargs['prompt'] = args.prompt
            errors.excepthook(sys.last_type, sys.last_value,
                              sys.last_traceback, **kwargs)
        except Exception as e:
            pass
        finally:
            errors.disable()

        # Insert suggested code into next cell.
        if args.p and CodeCompletionCache.get('last_completion'):
            self.shell.set_next_input(CodeCompletionCache.get('last_new_code'),
                                      replace=False)
        CodeCompletionCache.reset_class_vars()


get_ipython().register_magics(DebugMagic)
